@@ -4,20 +4,16 @@ import (
 	"fmt"
 )
 
+// CmdReload is `direnv reload`
 var CmdReload = &Cmd{
 	Name: "reload",
 	Desc: "triggers an env reload",
-	Fn: func(env Env, args []string) error {
-		config, err := LoadConfig(env)
-		if err != nil {
-			return err
-		}
-
+	Action: actionWithConfig(func(env Env, args []string, config *Config) error {
 		foundRC := config.FindRC()
 		if foundRC == nil {
 			return fmt.Errorf(".envrc not found")
 		}
 
 		return foundRC.Touch()
-	},
+	}),
 }
