@@ -5,18 +5,19 @@ import (
 	"path/filepath"
 )
 
+// CmdStatus is `direnv status`
 var CmdStatus = &Cmd{
 	Name: "status",
 	Desc: "prints some debug status information",
-	Fn: func(env Env, args []string) error {
-		config, err := LoadConfig(env)
-		if err != nil {
-			return err
-		}
-
+	Action: actionWithConfig(func(env Env, args []string, config *Config) error {
 		fmt.Println("direnv exec path", config.SelfPath)
 		fmt.Println("DIRENV_CONFIG", config.ConfDir)
-		fmt.Println("DIRENV_BASH", config.BashPath)
+
+		fmt.Println("bash_path", config.BashPath)
+		fmt.Println("disable_stdin", config.DisableStdin)
+		fmt.Println("warn_timeout", config.WarnTimeout)
+		fmt.Println("whitelist.prefix", config.WhitelistPrefix)
+		fmt.Println("whitelist.exact", config.WhitelistExact)
 
 		loadedRC := config.LoadedRC()
 		foundRC := config.FindRC()
@@ -34,7 +35,7 @@ var CmdStatus = &Cmd{
 		}
 
 		return nil
-	},
+	}),
 }
 
 func formatRC(desc string, rc *RC) {
