@@ -7,18 +7,22 @@ import (
 
 type elvish struct{}
 
-var ELVISH Shell = elvish{}
+// Elvish add support for the elvish shell
+var Elvish Shell = elvish{}
 
 func (elvish) Hook() (string, error) {
 	return `## hook for direnv
 @edit:before-readline = $@edit:before-readline {
 	try {
-		m = ("{{.SelfPath}}" export elvish | from-json)
-		keys $m | each [k]{
-			if (==s $k 'null') {
-				unset-env $k
-			} else {
-				set-env $k $m[$k]
+		m = [("{{.SelfPath}}" export elvish | from-json)]
+		if (> (count $m) 0) {
+			m = (explode $m)
+			keys $m | each [k]{
+				if $m[$k] {
+					set-env $k $m[$k]
+				} else {
+					unset-env $k
+				}
 			}
 		}
 	} except e {
